#include "abstract_tag.hpp"
#include <boost/format.hpp>
#include <boost/none.hpp>
#include <boost/lexical_cast.hpp>
using namespace TagMachine;
using namespace boost;
using namespace std;

namespace {
    map<string, string> parse_vorbis_tag(istream& is) {
        map<string, string> tags;
    
        string line;
        while (is.good()) {
            getline(is, line, '\n');

            // drop CR
            if (line.size() > 0 && line[line.size()-1] == '\r') {
                line.erase(line.size()-1);
            }

            // skip if this is not well formed
            string::size_type pos_eq = line.find('=');
            if (pos_eq == string::npos) {
                continue;
            }
            tags[line.substr(0, pos_eq)] = line.substr(pos_eq+1);
        }

        return tags;
    }
}

namespace TagMachine {
    boost::optional<tag_format_t>
    find_format_by_name(const std::string& str) {
    
        if (str == "vorbistag") {
            return TagMachine::TAG_VORBIS;
        }
        else {
            return none;
        }
    }

    tag_format_t
    force_find_format_by_name(const std::string& str) {
        optional<tag_format_t> fmt = find_format_by_name(str);

        if (fmt) {
            return *fmt;
        }
        else {
            throw (format("unknown tag file format: %s") % str).str();
        }
    }
}

AbstractTag::AbstractTag(istream& is, tag_format_t fmt) {
    if (fmt == TAG_VORBIS) {
        read_from_vorbis_tag(is);
    }
}

void AbstractTag::read_from_vorbis_tag(istream& is) {
    map<string, string> tags = parse_vorbis_tag(is);

    for (map<string, string>::const_iterator ite = tags.begin();
         ite != tags.end(); ite++) {

        if (ite->first == "TITLE") {
            title = ite->second;
        }
        else if (ite->first == "ARTIST") {
            artist = ite->second;
        }
        else if (ite->first == "ALBUM") {
            album = ite->second;
        }
        else if (ite->first == "COMMENT") {
            comment = ite->second;
        }
        else if (ite->first == "GENRE") {
            genre = ite->second;
        }
        else if (ite->first == "DATE") {
            // FIXME: this code is far from perfect.
            if (ite->second.size() == 4) {
                year = lexical_cast<int>(ite->second);
            }
        }
        else if (ite->first == "TRACKNUMBER") {
            tracknumber = lexical_cast<unsigned int>(ite->second);
        }
    }
}

void AbstractTag::write(ostream& os, tag_format_t fmt) {
    if (fmt == TAG_VORBIS) {
        write_to_vorbis_tag(os);
    }
}

void AbstractTag::write_to_vorbis_tag(ostream& os) {
    if (title) {
        os << "TITLE=" << *title << endl;
    }
    if (artist) {
        os << "ARTIST=" << *artist << endl;
    }
    if (album) {
        os << "ALBUM=" << *album << endl;
    }
    if (comment) {
        os << "COMMENT=" << *comment << endl;
    }
    if (genre) {
        os << "GENRE=" << *genre << endl;
    }
    if (year) {
        os << "DATE=" << *year << endl;
    }
    if (tracknumber) {
        os << "TRACKNUMBER=" << *tracknumber << endl;
    }
}
